proc Http_Proxy {{new {}}} {
     global http
     if ![info exists http(proxy)] {
         return {}
     }
     if {[string length $new] == 0} {
          return $http(proxy):$http(proxyPort)
          } else {
          regexp {^([^:]+):([0-9]+)$} $new x \
             http(proxy) http(proxyPort)
      }
}

proc Http_Open {url {command GET} {query {}}} {
     global http
     if {![regexp -nocase {^(http://)?([^:/]+)(:([0-9])+)?(/.*)} \
          $url x protocol server y port path]} {
          error "bogus URL: $url"
     }
     if {[string length $port] == 0} {
          set port 80
     }
     if {[info exists http(proxy)] &&
          [string length $http(proxy)]} {
       set sock [socket $http(proxy) $http(proxyPort)]
       puts $sock "$command http://$server:$port$path HTTP/
       1.0"
       } else {
       set sock [socket $server $port]
       puts $sock "$command $path HTTP/1.0"
       }
       puts $sock "User-Agent: Tcl/Tk Http_Open"
       puts $sock "Host: $server"
       if {[string length $query] > 0} {
           puts $sock "Content-Length: [string length $query]"
           puts $sock ""
           puts $sock $query
       }
       puts $sock ""
       flush $sock
       fconfigure $sock -blocking 0
       return $sock
 }



proc Http_Head {url} {
     upvar #0 $url state
     catch {unset state}
     set state(sock) [Http_Open $url HEAD]
     fileevent $state(sock) readable [list HttpHeader $url]
     # Specify the real name, not the upvar alias, to vwait
     vwait $url\(status)
     catch {close $state(status)}
}

proc HttpHeader {url} {
     upvar #0 $url state
     if [eof $state(sock)] {
     set state(status) eof
     close $state(sock)
     return
     }
     if [catch {gets $state(sock) line} nbytes] {
        set state(status) error
        lappend state(headers) [list error $nbytes]
        close $state(sock)
        return
        }
        if {$nbytes < 0} {
            #read would block
            return
      } elseif {$nbytes == 0} {
         #Header complete
         set state(status) head
      } elseif {![info exists state(headers)]} {
         # nitial status reply from the server
         set state(headers) [list http $line]
      } else {
         # Process key-value pairs
         regexp {^([^:]+): *(.*)$} $line x key value
         lappend state(headers) [string tolower $key] $value
     }
}

proc Http_Get {url {query {}}} {
     upvar #0 $url state       ;# Alias to global array
     catch {unset state}       ;# Aliases still valid.
     if {[string length $query] > 0} {
          set state(sock) [Http_Open $url POST $query]
     } else {
         set state(sock) [Http_Open $url GET]
     }
     set sock $state(sock)
     fileevent $sock readable [list HttpHeader $url]
     #specify the real name, not the upvar alias, to vwait
     vwait $url\(status)
     set header(content-type) {}
     set header(http) "500 unknown error"
     array set header $state(headers)
     # Check return status.
     # 200 is OK, other codes indicate a problem.
     regsub "HTTP/1.. " $header(http) {} header(http)
     if {![string match 2* $header(http)]} {
        catch {close $sock}
        if {[info exists header(location)] &&
             [string match 3* $header(http)]} {
             # 3xx is a redirection to another URL
             set state(link) $header(location)
             return [Http_Get $header(location) $query]
         }
         return -code error $header(http)
         }
         # Set up to read the content data
         switch -glob -- $header(content-type) {
            text/*   {
               # Read HTML into memory
               fileevent $sock readable [list HttpGetText $url]
               }
               default {
                    # Copy content data to a file
                    fconfigure $sock -translation binary
                    set state(filename) [File_TempName http]
                    if [catch {open $state(filename) w} out] {
                        set state(status) error
                        set state(error) $out
                        close $sock
                        return $header(content-type)
                    }
                    set state(fd) $out
                    fileevent $sock redable [list HttpCopyData $url]
                 }
             }
             vwait $url\(status)
             puts $state(body)
             return $header(content-type)
    }



proc HttpGetText {url} {
    upvar #0 $url state
    if [eof $state(sock)] {
       # Content complete
       set state(status) done
       close $state(sock)
       } elseif {[catch {read $state(sock)} block]} {
          set state(status) error
          lappend state(headers) [list error $block]
          close $state(sock)
       } else {
          append state(body) $block
       }
   }



rename unsupported0 copychannel


proc HttpCopyData {url} {
     upvar #0 $url state
     if [eof $state(sock)] {
         # Content complete
         set state(status) done
         close $state(sock)
         close $state(fd)
     } elseif {[catch {copychannel $state(sock) $state(fd)} x]} {
        set state(status) error
        lappend state(headers) [list error $x]
        close $state(sock)
        close $state(fd)
      }
 }




